@@ -16,11 +16,17 @@ class DemoApp:
 
     @api.field
     def set_preference_test(self, key: PreferenceKey, value: str) -> bool:
+        """Set a preference"""
         if isinstance(key, PreferenceKey):
             return True
         else:
             return False
 
+    @api.field
+    def get_preference_test(self) -> dict:
+        """Get a preference"""
+        return {"key": "ai_model", "value": "x"}
+
 
 mcp_server = GraphQLMCPServer(api=api)
 
